/*
 * PWM.c
 *
 *  Created on: 23 Jul 2019
 *      Author: kentm
 */

// ******************** includes **********************************************
#include <stdbool.h>
#include "PWM.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
// ******************** typedefs and defines **********************************

// ******************** local variables ***************************************

// ******************** local function prototypes *****************************

// ******************** Implementation - local helper functions ***************

// ******************** Implementation - exported functions - API *************
void PWMInit(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
  TIM_OCInitTypeDef  TIM_OCInitStructure;

	RCC_APB1PeriphClockCmd(TIMXRCC_APB1Periph, ENABLE);
  GPIO_PinAFConfig(TIM5_GPIO_PORT , TIM5CH1_Source , TIM5_AF_PORT);         //tim3 ch1
  GPIO_PinAFConfig(TIM5_GPIO_PORT , TIM5CH2_Source , TIM5_AF_PORT);         //tim3 ch2
  GPIO_PinAFConfig(TIM5_GPIO_PORT , TIM5CH3_Source , TIM5_AF_PORT);         //tim3 ch3
  GPIO_PinAFConfig(TIM5_GPIO_PORT , TIM5CH4_Source , TIM5_AF_PORT);         //tim3 ch4
  GPIO_PinAFConfig(TIM2_GPIO_PORT1 , TIM2CH1_Source , TIM2_AF_PORT);         //tim4 ch1
  GPIO_PinAFConfig(TIM2_GPIO_PORT2 , TIM2CH3_Source , TIM2_AF_PORT);         //tim4 ch2
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_High_Speed;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Mode =  GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Pin =  (1<<TIM2CH1_Source);
  GPIO_Init(TIM2_GPIO_PORT1, &GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = (1<<TIM2CH3_Source);
  GPIO_Init(TIM2_GPIO_PORT2, &GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = (1<<TIM5CH1_Source) | (1<<TIM5CH2_Source) | (1<<TIM5CH3_Source) | (1<<TIM5CH4_Source);
  GPIO_Init(TIM5_GPIO_PORT , &GPIO_InitStructure);

  TIM_TimeBaseStructure.TIM_Period = TIMER_PERIOD50HZ;
  TIM_TimeBaseStructure.TIM_Prescaler = TIMER_PRESCALER - 1; //timer tick 3Mhz = 96/32
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

  TIM_TimeBaseInit(TIM5, &TIM_TimeBaseStructure);
  TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = uS_to_ticks(RCPWM_NEUTRAL_uS);
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

  TIM_OC1Init(TIM2, &TIM_OCInitStructure);
  TIM_OC3Init(TIM2, &TIM_OCInitStructure);
  TIM_OC1Init(TIM5, &TIM_OCInitStructure);
  TIM_OC2Init(TIM5, &TIM_OCInitStructure);
  TIM_OC3Init(TIM5, &TIM_OCInitStructure);
  TIM_OC4Init(TIM5, &TIM_OCInitStructure);

  TIM_OC1PreloadConfig(TIM2, TIM_OCPreload_Enable);
  TIM_OC3PreloadConfig(TIM2, TIM_OCPreload_Enable);

  TIM_OC1PreloadConfig(TIM5, TIM_OCPreload_Enable);
  TIM_OC2PreloadConfig(TIM5, TIM_OCPreload_Enable);
  TIM_OC3PreloadConfig(TIM5, TIM_OCPreload_Enable);
  TIM_OC4PreloadConfig(TIM5, TIM_OCPreload_Enable);

  TIM_Cmd(TIM5, ENABLE);
  TIM_Cmd(TIM2, ENABLE);
}

void PWMSetFreq(TIM_TypeDef* TIMx, uint8_t ch,PWM_Freq_t PWM_Freq)
{
	static bool TIMx490Hz[2][5] = {0};
  uint8_t TimIdx = (TIM5==TIMx)?(0):(1);
  TIMx490Hz[TimIdx][ch] = PWM_Freq;
  for(int i=1;i<5;i++)
  {
  	if(TIMx490Hz[TimIdx][i] != PWM_Freq50Hz){PWM_Freq = PWM_Freq490Hz;i=10;}
  }
	TIM_Cmd(TIMx, DISABLE);
  TIM_SetAutoreload(TIMx, (PWM_Freq50Hz == PWM_Freq)?(TIMER_PERIOD50HZ):(TIMER_PERIOD490HZ));
  TIM_Cmd(TIMx, ENABLE);
}
void TIM_SetCompare(TIM_TypeDef* TIMx, uint8_t ch, uint32_t Compare)
{
	if(ch == 1)
	{
		TIMx->CCR1 = Compare;
		return;
	}
	if(ch == 2)
	{
		TIMx->CCR2 = Compare;
		return;
	}
	if(ch == 3)
	{
		TIMx->CCR3 = Compare;
		return;
	}
	if(ch == 4)
	{
		TIMx->CCR4 = Compare;
		return;
	}
}

// ******************** end of PWM.c ******************************************
