/*
 * Voters.c
 *
 *  Created on: 1 Aug 2019
 *      Author: kentm
 */

// ******************** includes **********************************************
#include "Voters.h"
// ******************** typedefs and defines **********************************
typedef struct {
	uint64_t UniqID;
	int8_t 	 SelectedFC;
	uint32_t LastTick;
} FCList_t;
#define MAXARBITERS	10
#define ARBITER_TIMEOUT_MS	 250																								// how long before arbiter is invalid
#define ARRAY_SIZE(arr) (sizeof(arr) / sizeof(arr[0]))
// ******************** local variables ***************************************
static FCList_t ActiveFCList[MAXARBITERS] = { [0 ... MAXARBITERS - 1].SelectedFC = -1 };
// ******************** local function prototypes *****************************
static bool InsertEntry(FCList_t *List, FCList_t *Entry, uint16_t ListLen, uint32_t TickmS);// insert a new entry, return false if could not insert
static int8_t FindMajorityEntry(FCList_t *List, uint16_t ListLen);
// ******************** Implementation - local helper functions ***************
////////////////////////////////////////////////////////////////////////////////
static bool InsertEntry(FCList_t *List, FCList_t *Entry, uint16_t ListLen, uint32_t TickmS)			// insert a new entry, return false if could not insert
{
	int8_t FreeEntry = -1;
	for (int8_t i = 0; i < ListLen; i++)
	{
		if ((FreeEntry < 0) && (List[i].SelectedFC < 0))
		{
			FreeEntry = i;
		}
		if((List[i].SelectedFC >= 0)&& (List[i].UniqID == Entry->UniqID))
		{
			List[i].SelectedFC = Entry->SelectedFC;
			List[i].LastTick = TickmS;																								// update time, for removing stale entries later
			return (true);
		}
	}
	// not found must ,make new entry
	if (FreeEntry < 0)
	{
		return (false);
	} 																																						// no room left
	Entry->LastTick = TickmS; 																										// update time, for removing stale entries later
	List[FreeEntry] = *Entry;
	return (true);
}
////////////////////////////////////////////////////////////////////////////////
static int8_t FindMajorityEntry(FCList_t *List, uint16_t ListLen)
{
	int8_t Vote[VoterCANPorts] = { 0 };
	int8_t Winner = -1;
	for (int8_t Entry = 0; Entry < ListLen; Entry++)															// check the list
	{
		int8_t Selected = List[Entry].SelectedFC;
		if((Selected>=0)&&(Selected < ARRAY_SIZE(Vote)))														// for valid selections
		{
			Vote[Selected]++;																													// and count votes for each port
		}
	}
	for (int8_t Port = VoterCANPorts - 1; Port >= 0; Port--)												// check each port from high to low
	{
		if (Vote[Port] > 0)																													// for any votes
		{
			if (Winner < 0)
			{
				Winner = Port;
			}																																					// if no winner yet, current is winner
			else if (Vote[Port] >= Vote[Winner])																			// else if >= winner make new winner
			{
				Winner = Port;
			}
		}
	}
	return (Winner);																															// return the winnner, ,ay be negative if no winner yet
}
// ******************** Implementation - exported functions - API *************
int8_t UpdateVoter(int8_t Port, uint8_t SrcBus, int8_t SelectedFC, int64_t UniqID,uint32_t TickMs)
{
	static int8_t LastRes = -1;
	int8_t Res = -1;
	if (Port != SrcBus)
	{
		return (LastRes);
	}
	static FCList_t Entry;
	Entry.SelectedFC = SelectedFC;
	Entry.UniqID = UniqID;
	if (InsertEntry(ActiveFCList, &Entry, ARRAY_SIZE(ActiveFCList),TickMs))
	{
		Res = FindMajorityEntry(ActiveFCList, ARRAY_SIZE(ActiveFCList));
	} else
	{
		Res = LastRes;
	}
	LastRes = Res;
	return (Res);
}
////////////////////////////////////////////////////////////////////////////////

bool UpdateVoterList50mS(int8_t* ActiveFCPort, int32_t TickmS)
{

	uint16_t ListLen = ARRAY_SIZE(ActiveFCList);
	FCList_t *List = ActiveFCList;
	bool Changed = false;
	for (int8_t i = 0; i < ListLen; i++)																					// parse list
	{
		if (List[i].SelectedFC >= 0)																								// if valid entry
		{
			if ((TickmS - List[i].LastTick) >= ARBITER_TIMEOUT_MS)												// if it has timed out
			{
				List[i].SelectedFC = -1;																								// invalidate entry
				List[i].UniqID = 0;
				List[i].LastTick = 0;
				Changed = true;																													// mark list change
			}
		}
	}
	if (Changed)																																	// if list changed
	{
		*ActiveFCPort = FindMajorityEntry(ActiveFCList, ARRAY_SIZE(ActiveFCList));		// scan list again and update active port
	}
	return(Changed);
}

// ******************** end of Voters.c ***************************************
