/*
 * serial.c
 *
 *  Created on: 15 Aug 2019
 *      Author: kentm
 */

// ******************** includes **********************************************
#include "stm32f4xx_ll_rcc.h"
#include "stm32f4xx_ll_bus.h"
#include "stm32f4xx_ll_system.h"
#include "stm32f4xx_ll_exti.h"
#include "stm32f4xx_ll_cortex.h"
#include "stm32f4xx_ll_utils.h"
#include "stm32f4xx_ll_pwr.h"
#include "stm32f4xx_ll_usart.h"
#include "stm32f4xx.h"
#include "stm32f4xx_ll_gpio.h"
#include "serial.h"
#include "que.h"
// ******************** typedefs and defines **********************************
#define RXQLEN	1024
#define TXQLEN	1024
// ******************** local variables ***************************************
static volatile bool tx_idle=true;
static uint8_t RxBuff[RXQLEN];
static uint8_t TxBuff[TXQLEN];
Q_t RxQ={.Buffer=RxBuff};
Q_t TxQ={.Buffer=TxBuff};
// ******************** local function prototypes *****************************

// ******************** Implementation - local helper functions ***************
static void MX_USART2_UART_Init(void)
{

  /* USER CODE BEGIN USART2_Init 0 */

  /* USER CODE END USART2_Init 0 */

  LL_USART_InitTypeDef USART_InitStruct = {0};

  LL_GPIO_InitTypeDef GPIO_InitStruct = {0};

  /* Peripheral clock enable */
  LL_APB1_GRP1_EnableClock(LL_APB1_GRP1_PERIPH_USART2);

  LL_AHB1_GRP1_EnableClock(LL_AHB1_GRP1_PERIPH_GPIOD);
  /**USART2 GPIO Configuration
  PD3   ------> USART2_CTS
  PD4   ------> USART2_RTS
  PD5   ------> USART2_TX
  PD6   ------> USART2_RX
  */
  GPIO_InitStruct.Pin = LL_GPIO_PIN_5|LL_GPIO_PIN_6;
  GPIO_InitStruct.Mode = LL_GPIO_MODE_ALTERNATE;
  GPIO_InitStruct.Speed = LL_GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.OutputType = LL_GPIO_OUTPUT_PUSHPULL;
  GPIO_InitStruct.Pull = LL_GPIO_PULL_UP;
  GPIO_InitStruct.Alternate = LL_GPIO_AF_7;
  LL_GPIO_Init(GPIOD, &GPIO_InitStruct);
  LL_GPIO_SetOutputPin(GPIOD,LL_GPIO_PIN_5);

  GPIO_InitStruct.Pin = LL_GPIO_PIN_3;
  GPIO_InitStruct.Mode = LL_GPIO_MODE_ALTERNATE;
  GPIO_InitStruct.Speed = LL_GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.OutputType = LL_GPIO_OUTPUT_PUSHPULL;
  GPIO_InitStruct.Pull = LL_GPIO_PULL_DOWN;
  GPIO_InitStruct.Alternate = LL_GPIO_AF_7;
  LL_GPIO_Init(GPIOD, &GPIO_InitStruct);

  GPIO_InitStruct.Pin = LL_GPIO_PIN_4;
  GPIO_InitStruct.Mode = LL_GPIO_MODE_ALTERNATE;
  GPIO_InitStruct.Speed = LL_GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.OutputType = LL_GPIO_OUTPUT_PUSHPULL;
  GPIO_InitStruct.Pull = LL_GPIO_PULL_NO;
  GPIO_InitStruct.Alternate = LL_GPIO_AF_7;
  LL_GPIO_Init(GPIOD, &GPIO_InitStruct);

  /* USART2 interrupt Init */
  NVIC_SetPriority(USART2_IRQn, NVIC_EncodePriority(NVIC_GetPriorityGrouping(),0, 0));
  NVIC_EnableIRQ(USART2_IRQn);

  /* USER CODE BEGIN USART2_Init 1 */

  /* USER CODE END USART2_Init 1 */
  USART_InitStruct.BaudRate = 230400;
  USART_InitStruct.DataWidth = LL_USART_DATAWIDTH_8B;
  USART_InitStruct.StopBits = LL_USART_STOPBITS_1;
  USART_InitStruct.Parity = LL_USART_PARITY_NONE;
  USART_InitStruct.TransferDirection = LL_USART_DIRECTION_TX_RX;
  USART_InitStruct.HardwareFlowControl = LL_USART_HWCONTROL_RTS_CTS;
  USART_InitStruct.OverSampling = LL_USART_OVERSAMPLING_16;
  LL_USART_Init(USART2, &USART_InitStruct);
  LL_USART_ConfigAsyncMode(USART2);
  LL_USART_Enable(USART2);
  /* USER CODE BEGIN USART2_Init 2 */

  /* USER CODE END USART2_Init 2 */

}
// ******************** Implementation - exported functions - API *************

bool InitSerial(void)
{
	bool Res= true;
	VarInitQ(&TxQ,sizeof(TxBuff));
	VarInitQ(&RxQ,sizeof(RxBuff));
	MX_USART2_UART_Init();
	LL_USART_EnableIT_RXNE(USART2);
	LL_USART_EnableIT_TC(USART2);
	return(Res);
}
////////////////////////////////////////////////////////////////////////////////
void USART2_IRQHandler(void)
{
	if(LL_USART_IsActiveFlag_RXNE(USART2))
	{
		static uint8_t Byte;
		Byte = LL_USART_ReceiveData8(USART2);
		EnQ1(&RxQ,Byte);
		LL_USART_ClearFlag_RXNE(USART2);
	}
	if(LL_USART_IsActiveFlag_TC(USART2))
	{
		static uint8_t TxByte;
		LL_USART_ClearFlag_TC(USART2);
		if(DeQ1(&TxQ,&TxByte))	// pull the next available data from the queue
		{
			LL_USART_TransmitData8(USART2,TxByte);
		}
		else
		{
			tx_idle = true;
		}
	}
}
////////////////////////////////////////////////////////////////////////////////
uint16_t SerialTxQ(uint8_t *Data, uint16_t len)
{
	bool Res = EnQ (&TxQ,Data,len);		// queue data block of len bytes
	if (tx_idle)
	{
		uint8_t TxByte;
		if(DeQ1(&TxQ,&TxByte))	// pull the next available data from the queue
		{
			tx_idle = false;
			LL_USART_TransmitData8(USART2,TxByte);
		}
	}
	return(Res?len:0);
}
////////////////////////////////////////////////////////////////////////////////
uint16_t SerialTxQFree(void)										// returns tx q free space
{
	uint16_t len;
	len = QFree(&TxQ);
	return(len);

}
////////////////////////////////////////////////////////////////////////////////
bool SerialTxQMT(void)										// returns if anything in tx q
{
	return(QMT(&TxQ));
}
////////////////////////////////////////////////////////////////////////////////
uint16_t SerialTxQWait(uint8_t *Data, uint16_t len)
{
	uint16_t sent=SerialTxQ(Data,len);
	while(!SerialTxQMT());
	return(sent);
}
////////////////////////////////////////////////////////////////////////////////
bool SerialRxQ(uint8_t *byte)								// returns true if data present
{
	return(DeQ1(&RxQ,byte));
}
////////////////////////////////////////////////////////////////////////////////
bool SerialRxQBuff(uint8_t *Data, uint16_t *len)	//	true if data read, pass space in len and returns amount in len
{
	return(DeQ(&RxQ,Data,len));
}
////////////////////////////////////////////////////////////////////////////////
uint16_t SerialRxQSize(void)								// returns amount of data in RX queue
{
	return(QLen(&RxQ));								// get the length of data in q (without dequeuing)
}

// ******************** end of serial.c *********************************** 	
